/*
    This program demonstrates the Windows API for creating processes,
    the CreateProcess() function. To help you understand what is
    being done by this code, look up the following terms in MSDN.
       CreateProcess
       STARTUPINFO
       PROCESS_INFORMATION
       ZeroMemory
       CloseHandle
       LPTSTR
       GetLastError
       FormatMessage
*/
#include <stdio.h>
#include <windows.h>

// function prototype
void printError(char* functionName);

int main()
{
   int i;

   #define NUMBER_OF_PROCESSES 3
   LPTSTR lpCommandLine[NUMBER_OF_PROCESSES];
   PROCESS_INFORMATION processInfo[NUMBER_OF_PROCESSES];

   STARTUPINFO startInfo;
   ZeroMemory(&startInfo, sizeof(startInfo));
   startInfo.cb = sizeof(startInfo);

   /* set up the command lines */
   lpCommandLine[0] = "C:\\Windows\\notepad.exe";
   lpCommandLine[1] = "C:\\Windows\\system32\\calc.exe";
   lpCommandLine[2] = "C:\\Windows\\System32\\nslookup.exe";

   /* create the processes */
   for (i = 0; i < NUMBER_OF_PROCESSES; i++)
   {
      if( !CreateProcess(NULL, lpCommandLine[i], NULL, NULL, FALSE,
                         HIGH_PRIORITY_CLASS | CREATE_NEW_CONSOLE,
                         NULL, NULL, &startInfo, &processInfo[i]) )
      {
          printError("CreateProcess");
      }
      else
      {
         printf("Started program %d with pid = %d\n\n", i, (int)processInfo[i].dwProcessId);
      }
   }

   /* close all the handles */
   for (i = 0; i < NUMBER_OF_PROCESSES; i++)
   {
      CloseHandle(processInfo[i].hThread);
      CloseHandle(processInfo[i].hProcess);
   }

   return 0;
}



/****************************************************************
   The following function can be used to print out "meaningfull"
   error messages. If you call a Win32 function and it returns
   with an error condition, then call this function right away and
   pass it a string containing the name of the Win32 function that
   failed. This function will print out a reasonable text message
   explaining the error and then (if chosen) terminate the program.
*/
void printError(char* functionName)
{
   LPVOID lpMsgBuf;
    int error_no;
    error_no = GetLastError();
    FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
    );
    // Display the string.
    fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
    fprintf(stderr, (const char*)lpMsgBuf);
    // Free the buffer.
    LocalFree( lpMsgBuf );
    //ExitProcess(1);  // terminate the program
}//printError
