/*
   The fork() system call is the mechanism for creating new
   processes in Linux/Unix/Posix API. It creates an exact
   duplicate of the current process (the duplicate is the
   child process and the original process is the parent).
   If you want a really "different" process, then the child
   process should use a version of the exec function to replace
   this program's code with the code of the new program.

   This program creates four processes (why?). The four processes
   will be related by the following "family tree".

                    parent
                    /    \
                  /        \
               child1     child2
                 |
                 |
               child

   Try to identify (in terms of parents and children) which process
   created each output.
*/
#include <stdio.h>  // printf(), fprintf()
#include <stdlib.h> // exit()
#include <string.h> // strerror()
#include <errno.h>  // errno
#include <unistd.h> // fork()

// function prototype
void printError(char* functionName);

int main()
{
   int pid1, pid2;

   pid1 = fork();  // child gets pid1 = 0, parent gets pid1 = child's pid
   if (pid1 == -1 ) printError("fork");

   pid2 = fork();
   if (pid2 == -1 ) printError("fork");

   printf("pid1 = %d and pid2 = %d (so who am I in the family tree?)\n", pid1, pid2);

   exit(0);
}


/****************************************************************
   The following function can be used to print out "meaningfull"
   error messages. If you call a Unix function and it returns
   with an error condition, then call this function right away and
   pass it a string containing the name of the Unix function that
   failed. This function will print out a reasonable text message
   explaining the error and then terminate the program.
*/
void printError(char* functionName) /* unix-style error */
{
   fprintf(stderr, "%s: %s (errno = %d)\n", functionName, strerror(errno), errno);
   exit(0);
}
