/*  prompting shell version 3
 *		Solves the `one-shot' problem of version 1
 *			Uses execvp(), but fork()s first so that the
 *			shell waits around to perform another command
 *			CTRL-D will close this shell
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include	<sys/wait.h>

void execute( char *[] );
char * makestring( char * );

#define	MAXARGS		20			/* cmdline args	*/
#define	ARGLEN		100			/* token length	*/

int main()
{
	char	*arglist[MAXARGS+1];		/* an array of ptrs	*/
	int	numargs;			/* index into array	*/
	char	argbuf[ARGLEN];			/* read stuff here	*/
	char	*makestring();			/* malloc etc		*/

	numargs = 0;
	while ( numargs < MAXARGS )
	{	/*use can type either an argument, ENTER, or CTRL-D */
		printf("Arg[%d]? ", numargs);
		if ( fgets(argbuf, ARGLEN, stdin) ){
			if ( *argbuf != '\n' ) /* user typed a command name */
				arglist[numargs++] = makestring(argbuf);
			else /* user typed ENTER */
			{
				if ( numargs > 0 ){		/* any args?	*/
					arglist[numargs]=NULL;	/* close list	*/
					execute( arglist );	/* do it	*/
					numargs = 0;		/* and reset	*/
				}
			}
		}
		else /* user typed CTRL-D */
		{
			if ( numargs > 0 ){		/* any args?	*/
				arglist[numargs]=NULL;	/* close list	*/
				execute( arglist );	/* do it	*/
			}
			break;
		}
	}
	return 0;
}


/*
 *	use fork and execvp and wait to do it
 */
void execute( char *arglist[] )
{
	int	pid,exitstatus;			/* of child	*/

	pid = fork();				/* make new process */
	switch( pid ){
		case -1:	
			perror("fork failed");
			exit(1);
		case 0:
			execvp(arglist[0], arglist);		/* do it */
			perror("execvp failed");
			exit(1);
		default:
			while( wait(&exitstatus) != pid )
				;
			printf("child exited with status %d,%d\n",
					exitstatus>>8, exitstatus&0377);
	}
}


/*
 * trim off newline and create storage for the string
 */
char * makestring( char *buf )
{
	char	*cp;

	buf[strlen(buf)-1] = '\0';		/* trim newline	*/
	cp = malloc( strlen(buf)+1 );		/* get memory	*/ /*** THIS MEMORY IS NEVER FREED ***/
	if ( cp == NULL ){			/* or die	*/
		fprintf(stderr,"no memory\n");
		exit(1);
	}
	strcpy(cp, buf);		/* copy chars	*/
	return cp;			/* return ptr	*/
}
