/*  prompting shell version 5
 *    Solves the `one-shot' problem of version 1
 *       Uses execvp(), but fork()s first so that the
 *       shell waits around to perform another command
 *       CTRL-D will close this shell
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>

char ** next_cmd();
void execute( char *[] );
char * makestring( char * );

#define  MAXARGS     20       /* cmdline args   */
#define  ARGLEN      100         /* token length   */

int main() /* this main is written more like smsh1.c in Chapter 9 */
{
   char  **arglist;        /* an array of ptrs */

   while ( (arglist = next_cmd()) != NULL )
   {
      if ( arglist[0] != NULL ){ /* any args? */
         execute( arglist );  /* do it */
         char **cpp = arglist;   /* free the argument list */
         while ( *cpp )
            free( *cpp++ );
         free( arglist );
      }
   }
   return 0;
}


/*
 * purpose: read next command line
 * returns: dynamically allocated array holding command line arguments
 *        : NULL at EOF
 */
char ** next_cmd()
{
   char  **arglist;        /* an array of ptrs */
   int   numargs;       /* index into array */
   char  argbuf[ARGLEN];         /* read stuff here */
   char  *makestring();       /* malloc etc */

   /* get arglist[0] */
   /* user can type either a command name, ENTER, or CTRL-D */
   numargs = 0;
   printf("Arg[%d]? ", numargs);
   if ( fgets(argbuf, ARGLEN, stdin) )
   {
      arglist = malloc( sizeof(char*) * (MAXARGS+1) );
      if ( *argbuf != '\n' ) /* user typed a command name */
         arglist[numargs++] = makestring(argbuf);
      else /* user typed ENTER */
      {
         arglist[0] = NULL;
         return arglist;
      }
   }
   else /* user typed CTRL-D (EOF) */
      return NULL;

   /* get the rest of the arglist */
   while ( numargs < MAXARGS )
   {
      printf("Arg[%d]? ", numargs);
      if ( fgets(argbuf, ARGLEN, stdin) && *argbuf != '\n' )
         arglist[numargs++] = makestring(argbuf);
      else
      {
         arglist[numargs] = NULL;   /* close list */
         return arglist;
      }
   }
   arglist[MAXARGS] = NULL;   /* close list */
   return arglist;
}


/*
 * use fork and execvp and wait to do it
 */
void execute( char *arglist[] )
{
   int   pid,exitstatus;         /* of child */

   pid = fork();           /* make new process */
   switch( pid ){
      case -1:
         perror("fork failed");
         exit(1);
      case 0:
         execvp(arglist[0], arglist);     /* do it */
         perror("execvp failed");
         exit(1);
      default:
         while( wait(&exitstatus) != pid )
            ;
         printf("child exited with status %d,%d\n",
               exitstatus>>8, exitstatus&0377);
   }
}


/*
 * trim off newline and create storage for the string
 */
char * makestring( char *buf )
{
   char  *cp;

   buf[strlen(buf)-1] = '\0';    /* trim newline   */
   cp = malloc( strlen(buf)+1 );    /* get memory  */
   if ( cp == NULL ){         /* or die   */
      fprintf(stderr,"no memory\n");
      exit(1);
   }
   strcpy(cp, buf);     /* copy chars  */
   return cp;        /* return ptr  */
}
